@@ -169,7 +169,6 @@ static int function_ZeroCrossingsDASSL(int *neqm, double *t, double *y,
 int dassl_initial(DATA* data, threadData_t *threadData,
                   SOLVER_INFO* solverInfo, DASSL_DATA *dasslData)
 {
-  TRACE_PUSH
   /* work arrays for DASSL */
   unsigned int i;
   long N;
@@ -283,7 +282,7 @@ int dassl_initial(DATA* data, threadData_t *threadData,
   if (omc_flag[FLAG_NOEQUIDISTANT_GRID])
   {
     dasslData->dasslSteps = 1; /* TRUE */
-    solverInfo->solverNoEquidistantGrid = 1;
+    solverInfo->solverNoEquidistantGrid = TRUE;
   }
   else
   {
@@ -322,7 +321,7 @@ int dassl_initial(DATA* data, threadData_t *threadData,
   {
     for(i=1; i< JAC_MAX;i++)
     {
-      if(!strcmp((const char*)omc_flagValue[FLAG_JACOBIAN], JACOBIAN_METHOD[i])){
+      if(!strcmp((const char*)omc_flagValue[FLAG_JACOBIAN], JACOBIAN_METHOD_NAME[i])){
         dasslData->dasslJacobian = (int)i;
         break;
       }
@@ -334,7 +333,7 @@ int dassl_initial(DATA* data, threadData_t *threadData,
         warningStreamPrint(OMC_LOG_SOLVER, 1, "unrecognized jacobian calculation method %s, current options are:", (const char*)omc_flagValue[FLAG_JACOBIAN]);
         for(i=1; i < JAC_MAX; ++i)
         {
-          warningStreamPrint(OMC_LOG_SOLVER, 0, "%-15s [%s]", JACOBIAN_METHOD[i], JACOBIAN_METHOD_DESC[i]);
+          warningStreamPrint(OMC_LOG_SOLVER, 0, "%-15s [%s]", JACOBIAN_METHOD_NAME[i], JACOBIAN_METHOD_DESC[i]);
         }
         messageClose(OMC_LOG_SOLVER);
       }
@@ -347,12 +346,12 @@ int dassl_initial(DATA* data, threadData_t *threadData,
     dasslData->dasslJacobian = COLOREDNUMJAC;
   }
 
-  ANALYTIC_JACOBIAN* jacobian = &(data->simulationInfo->analyticJacobians[data->callback->INDEX_JAC_A]);
+  JACOBIAN* jacobian = &(data->simulationInfo->analyticJacobians[data->callback->INDEX_JAC_A]);
   data->callback->initialAnalyticJacobianA(data, threadData, jacobian);
   if(jacobian->availability == JACOBIAN_AVAILABLE || jacobian->availability == JACOBIAN_ONLY_SPARSITY) {
     infoStreamPrint(OMC_LOG_SIMULATION, 1, "Initialized Jacobian:");
-    infoStreamPrint(OMC_LOG_SIMULATION, 0, "columns: %d rows: %d", jacobian->sizeCols, jacobian->sizeRows);
-    infoStreamPrint(OMC_LOG_SIMULATION, 0, "NNZ:  %d colors: %d", jacobian->sparsePattern->numberOfNonZeros, jacobian->sparsePattern->maxColors);
+    infoStreamPrint(OMC_LOG_SIMULATION, 0, "columns: %zu rows: %zu", jacobian->sizeCols, jacobian->sizeRows);
+    infoStreamPrint(OMC_LOG_SIMULATION, 0, "NNZ:  %u colors: %u", jacobian->sparsePattern->numberOfNonZeros, jacobian->sparsePattern->maxColors);
     messageClose(OMC_LOG_SIMULATION);
   }
 
@@ -433,7 +432,6 @@ int dassl_initial(DATA* data, threadData_t *threadData,
   /* ### end configuration of dassl ### */
 
   messageClose(OMC_LOG_SOLVER);
-  TRACE_POP
   return 0;
 }
 
@@ -443,7 +441,6 @@ int dassl_initial(DATA* data, threadData_t *threadData,
  */
 int dassl_deinitial(DATA* data, DASSL_DATA *dasslData)
 {
-  TRACE_PUSH
   unsigned int i;
 
   /* free work arrays for DASSL */
@@ -462,8 +459,8 @@ int dassl_deinitial(DATA* data, DASSL_DATA *dasslData)
   free(dasslData->states);
 
   /* Free Jacobians */
-  ANALYTIC_JACOBIAN* jacobian = &(data->simulationInfo->analyticJacobians[data->callback->INDEX_JAC_A]);
-  freeAnalyticJacobian(jacobian);
+  JACOBIAN* jacobian = &(data->simulationInfo->analyticJacobians[data->callback->INDEX_JAC_A]);
+  freeJacobian(jacobian);
 
 #ifdef USE_PARJAC
   if (dasslData->allocatedParMem) {
@@ -474,7 +471,6 @@ int dassl_deinitial(DATA* data, DASSL_DATA *dasslData)
 
   free(dasslData);
 
-  TRACE_POP
   return 0;
 }
 
@@ -534,7 +530,6 @@ int printVector(int logLevel, const char* name,  double* vec, int n, double time
  **********************************************************************************************/
 int dassl_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo)
 {
-  TRACE_PUSH
   double tout = 0;
   int i = 0;
   unsigned int ui = 0;
@@ -575,7 +570,6 @@ int dassl_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo)
   /* If an event is triggered and processed restart dassl. */
   if(!dasslData->dasslAvoidEventRestart && (solverInfo->didEventStep || 0 == dasslData->idid))
   {
-    debugStreamPrint(OMC_LOG_EVENTS_V, 0, "Event-management forced reset of DDASKR");
     /* obtain reset */
     dasslData->info[0] = 0;
     dasslData->idid = 0;
@@ -667,7 +661,6 @@ int dassl_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo)
         fflush(stdout);
         retVal = continue_DASSL(&dasslData->idid, &data->simulationInfo->tolerance);
         warningStreamPrint(OMC_LOG_STDOUT, 0, "can't continue. time = %f", sData->timeValue);
-        TRACE_POP
         break;
       }
       else if(dasslData->idid == 5)
@@ -708,7 +701,6 @@ int dassl_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo)
 
   if (return_from_small_step) {
     /* need to do this outside the try-catch macro */
-    TRACE_POP
     return 0;
   }
 
@@ -725,7 +717,7 @@ int dassl_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo)
   solverInfo->solverStatsTmp.nCallsODE                = dasslData->iwork[11];
   solverInfo->solverStatsTmp.nCallsJacobian           = dasslData->iwork[12];
   solverInfo->solverStatsTmp.nErrorTestFailures       = dasslData->iwork[13];
-  solverInfo->solverStatsTmp.nConvergenveTestFailures = dasslData->iwork[14];
+  solverInfo->solverStatsTmp.nConvergenceTestFailures = dasslData->iwork[14];
 
   if(OMC_ACTIVE_STREAM(OMC_LOG_DASSL))
   {
@@ -740,21 +732,19 @@ int dassl_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo)
     infoStreamPrint(OMC_LOG_DASSL, 0, "number of steps taken so far: %d", solverInfo->solverStatsTmp.nStepsTaken);
     infoStreamPrint(OMC_LOG_DASSL, 0, "number of calls of functionODE() : %d", solverInfo->solverStatsTmp.nCallsODE);
     infoStreamPrint(OMC_LOG_DASSL, 0, "number of calculation of jacobian : %d", solverInfo->solverStatsTmp.nCallsJacobian);
-    infoStreamPrint(OMC_LOG_DASSL, 0, "total number of convergence test failures: %d", solverInfo->solverStatsTmp.nErrorTestFailures);
-    infoStreamPrint(OMC_LOG_DASSL, 0, "total number of error test failures: %d", solverInfo->solverStatsTmp.nConvergenveTestFailures);
+    infoStreamPrint(OMC_LOG_DASSL, 0, "total number of convergence test failures: %d", solverInfo->solverStatsTmp.nConvergenceTestFailures);
+    infoStreamPrint(OMC_LOG_DASSL, 0, "total number of error test failures: %d", solverInfo->solverStatsTmp.nErrorTestFailures);
     messageClose(OMC_LOG_DASSL);
   }
 
   infoStreamPrint(OMC_LOG_DASSL, 0, "Finished DASSL step.");
   if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
 
-  TRACE_POP
   return retVal;
 }
 
 static int continue_DASSL(int* idid, double* atol)
 {
-  TRACE_PUSH
   int retValue = -1;
 
   switch(*idid)
@@ -811,7 +801,6 @@ static int continue_DASSL(int* idid, double* atol)
     break;
   }
 
-  TRACE_POP
   return retValue;
 }
 
@@ -835,7 +824,6 @@ static int continue_DASSL(int* idid, double* atol)
 static int functionODE_residual(double *t, double *y, double *yd, double* cj,
                                 double *delta, int *ires, double *rpar, int *ipar)
 {
-  TRACE_PUSH
   UNUSED(cj); UNUSED(ipar); /* Silence compíler warnings */
 
   DATA* data = (DATA*)((double**)rpar)[0];
@@ -903,14 +891,12 @@ static int functionODE_residual(double *t, double *y, double *yd, double* cj,
   if (measure_time_flag) rt_accumulate(SIM_TIMER_RESIDUALS);
   if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
 
-  TRACE_POP
   return 0;
 }
 
 static int function_ZeroCrossingsDASSL(int *neqm, double *t, double *y, double *yp,
                                        int *ng, double *gout, double *rpar, int* ipar)
 {
-  TRACE_PUSH
   DATA* data = (DATA*)(void*)((double**)rpar)[0];
   DASSL_DATA* dasslData = (DASSL_DATA*)(void*)((double**)rpar)[1];
   threadData_t *threadData = (threadData_t*)(void*)((double**)rpar)[2];
@@ -950,7 +936,6 @@ static int function_ZeroCrossingsDASSL(int *neqm, double *t, double *y, double *
   if (measure_time_flag) rt_accumulate(SIM_TIMER_EVENT);
   if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
 
-  TRACE_POP
   return 0;
 }
 
@@ -984,22 +969,20 @@ int jacA_symColored(double *t, double *y, double *yprime, double *delta,
                     double *matrixA, double *cj, double *h, double *wt,
                     double *rpar, int *ipar)
 {
-  TRACE_PUSH
   DATA* data = (DATA*)(void*)((double**)rpar)[0];
   threadData_t *threadData = (threadData_t*)(void*)((double**)rpar)[2];
   DASSL_DATA* dasslData = (DASSL_DATA*)(void*)((double**)rpar)[1];
   const int index = data->callback->INDEX_JAC_A;
-  ANALYTIC_JACOBIAN* jac = &(data->simulationInfo->analyticJacobians[index]);
+  JACOBIAN* jac = &(data->simulationInfo->analyticJacobians[index]);
 
 #ifdef USE_PARJAC
-  ANALYTIC_JACOBIAN* t_jac = (dasslData->jacColumns);
+  JACOBIAN* t_jac = (dasslData->jacColumns);
 #else
-  ANALYTIC_JACOBIAN* t_jac = jac;
+  JACOBIAN* t_jac = jac;
 #endif
 
   unsigned int columns = jac->sizeCols;
   unsigned int rows = jac->sizeRows;
-  unsigned int sizeTmpVars = jac->sizeTmpVars;
   SPARSE_PATTERN* spp = jac->sparsePattern;
 
   /* Evaluate constant equations if available */
@@ -1010,7 +993,6 @@ int jacA_symColored(double *t, double *y, double *yprime, double *delta,
   genericColoredSymbolicJacobianEvaluation(rows, columns, spp, matrixA, t_jac,
                                            data, threadData, &setJacElementDasslSparse);
 
-  TRACE_POP
   return 0;
 }
 
@@ -1025,14 +1007,13 @@ int jacA_sym(double *t, double *y, double *yprime, double *delta,
              double *matrixA, double *cj, double *h, double *wt, double *rpar,
              int *ipar)
 {
-  TRACE_PUSH
 
   DATA* data = (DATA*)(void*)((double**)rpar)[0];
   DASSL_DATA* dasslData = (DASSL_DATA*)(void*)((double**)rpar)[1];
   threadData_t *threadData = (threadData_t*)(void*)((double**)rpar)[2];
 
   const int index = data->callback->INDEX_JAC_A;
-  ANALYTIC_JACOBIAN* jac = &(data->simulationInfo->analyticJacobians[index]);
+  JACOBIAN* jac = &(data->simulationInfo->analyticJacobians[index]);
   unsigned int columns = jac->sizeCols;
   unsigned int rows = jac->sizeRows;
   unsigned int sizeTmpVars = jac->sizeTmpVars;
@@ -1058,10 +1039,10 @@ int jacA_sym(double *t, double *y, double *yprime, double *delta,
      GC_register_my_thread (&sb);
   }
   // Use thread local analytic Jacobians
-  ANALYTIC_JACOBIAN* t_jac = &(dasslData->jacColumns[omc_get_thread_num()]);
-  //printf("index= %d, t_jac->sizeCols= %d, t_jac->sizeRows = %d, t_jac->sizeTmpVars = %d \n",index, t_jac->sizeCols , t_jac->sizeRows, t_jac->sizeTmpVars);
+  JACOBIAN* t_jac = &(dasslData->jacColumns[omc_get_thread_num()]);
+  //printf("index= %d, t_jac->sizeCols= %d, t_jac->sizeRows = %d, t_jac->sizeTmpVars = %d\n",index, t_jac->sizeCols , t_jac->sizeRows, t_jac->sizeTmpVars);
 #else
-  ANALYTIC_JACOBIAN* t_jac = jac;
+  JACOBIAN* t_jac = jac;
 #endif
   unsigned int j;
 
@@ -1080,7 +1061,6 @@ int jacA_sym(double *t, double *y, double *yprime, double *delta,
   } // for loop
 } // omp parallel
 
-  TRACE_POP
   return 0;
 }
 
@@ -1108,7 +1088,6 @@ int jacA_num(double *t, double *y, double *yprime, double *delta,
              double *matrixA, double *cj, double *h, double *wt, double *rpar,
              int *ipar)
 {
-  TRACE_PUSH
   DATA* data = (DATA*)(void*)((double**)rpar)[0];
   DASSL_DATA* dasslData = (DASSL_DATA*)(void*)((double**)rpar)[1];
   threadData_t* threadData = (threadData_t*)(void*)((double**)rpar)[2];
@@ -1145,7 +1124,6 @@ int jacA_num(double *t, double *y, double *yprime, double *delta,
     y[col] = ysave;
   }
 
-  TRACE_POP
   return 0;
 }
 
@@ -1173,14 +1151,13 @@ int jacA_numColored(double *t, double *y, double *yprime, double *delta,
                     double *matrixA, double *cj, double *h, double *wt,
                     double *rpar, int *ipar)
 {
-  TRACE_PUSH
 
   DATA* data = (DATA*)(void*)((double**)rpar)[0];
   DASSL_DATA* dasslData = (DASSL_DATA*)(void*)((double**)rpar)[1];
   threadData_t *threadData = (threadData_t*)(void*)((double**)rpar)[2];
 
   const int index = data->callback->INDEX_JAC_A;
-  ANALYTIC_JACOBIAN* jacobian = &(data->simulationInfo->analyticJacobians[index]);
+  JACOBIAN* jacobian = &(data->simulationInfo->analyticJacobians[index]);
 
   double delta_h = numericalDifferentiationDeltaXsolver;
   double delta_hhh;
@@ -1232,7 +1209,6 @@ int jacA_numColored(double *t, double *y, double *yprime, double *delta,
     }
   }
 
-  TRACE_POP
   return 0;
 }
 
@@ -1258,7 +1234,6 @@ static int callJacobian(double *t, double *y, double *yprime, double *deltaD,
                         double *pd, double *cj, double *h, double *wt,
                         double *rpar, int* ipar)
 {
-  TRACE_PUSH
   DATA* data = (DATA*)(void*)((double**)rpar)[0];
   DASSL_DATA* dasslData = (DASSL_DATA*)(void*)((double**)rpar)[1];
   threadData_t *threadData = (threadData_t*)(void*)((double**)rpar)[2];
@@ -1272,7 +1247,6 @@ static int callJacobian(double *t, double *y, double *yprime, double *deltaD,
   if(dasslData->jacobianFunction(t, y, yprime, deltaD, pd, cj, h, wt, rpar, ipar))
   {
     throwStreamPrint(threadData, "Error, can not get Matrix A ");
-    TRACE_POP
     return 1;
   }
 
@@ -1296,7 +1270,6 @@ static int callJacobian(double *t, double *y, double *yprime, double *deltaD,
   rt_accumulate(SIM_TIMER_JACOBIAN);
   if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
 
-  TRACE_POP
   return 0;
 }
 
